[§1] Sagittarius extensions {#ext.sagittarius}
=============

Sagittarius has its own extension libraries because even R6RS is huge however I
know it is not  sufficient to write practical program. To  support to write it,
Sagittarius provides some useful libraries.

[§2] (sagittarius) - builtin library {#lib.sagittarius}
-------------

###### [!Library] `(sagittarius)` 

This library has Sagittarius specific functionalities such as extra file
system functions and so.

### [§3] Builtin Syntax {#lib.sagittarius.builtin.syntax}

###### [!Syntax] `define-constant`  _variable_ _expression_

Similar to the `define` however the `define-constant` binds
_variable_ as a constant value and the compiler try to fold it if it is
constant value i.e. literal string, literal vector, literal number and so.

If the defined _variable_ is being overwritten, the VM shows a warning
message on the standard error. If overwriten is forbidden by
`#!no-overwite` or other directives, then compiler raises an error.


###### [!Syntax] `receive`  _formals_ _expression_ _body_

[SRFI-8] _formals_ and _body_ are the same as `lambda`.
_Expression_ must be an expression.

`receive` binds values which are generated by _expressions_ to
_formals_.

The expressions in _body_ are evaluated sequentially in the extended
environment. The results of the last expression in the body are the values of
the `receive`-expression.


###### [!Syntax] `cond-expand`  _clauses_ _..._

[R7RS][SRFI-0] Compile time condition. The `cond-expand` resolves
platform dependencies such as C's `#ifdef` preprocessor.

_clauses_ must be one of these forms:

- (_feature-identifier_ _body_ ...)
- ((`library` _library-name_) _body_ ...)
- ((`and` _feature-identifier_ ...) _body_ ...)
- ((`or` _feature-identifier_ ...) _body_ ...)
- (`not` _feature-identifier_)

`library` form searches the given _library-name_ and if it is found,
then compiles _body_.

`and`, `or` and `not` are the same as usual syntax.

Users can check possible _feature-identifier_s via `cond-features`procedure.


### [§3] Macro transformer

###### [!Function] `er-macro-transformer`  _proc_

_Proc_ must be a procedure which accepts 3 arguments, _form_,
_rename_ and _compare_.

_form_
: The input form of this macro. It is mere s-expression.

_rename_
: A procedure. It takes an object as its argument and
  convert it to a syntax object. The procedure is similar with 
  `datum->syntax`, it doesn't require template identifier.

_compare_
: A procedure. It takes 2 arguments and compares if they
  are the same object in sense of `free-identifier=?`. If the given
  arguments are list or vector, then the procedure checks its elements.

The `er-macro-transformer` returns explicit renaming macro transformer, so
you can write both hygine and non-hygine macro with it. For example:

``````````scheme
(define-syntax loop
  (er-macro-transformer
   (lambda (form rename compare)
     (let ((body (cdr form)))
       `(,(rename 'call/cc)
	 (,(rename 'lambda) (break)
	  (,(rename 'let) ,(rename 'f) () ,@body (,(rename 'f)))))))))

(let ((n 3) (ls '()))
  (loop
    (if (= n 0) (break ls))
    (set! ls (cons 'a ls))
    (set! n (- n 1))))
``````````
=> ``(a a a)``

This example has the same functionality as the example written in
`datum->syntax` description. The basic of `er-macro-transformer` is
the opposite way of the `syntax-case`. The `syntax-case` always makes
macro hygine, however the `er-macro-transformer` can make macro hygine.
Moreover, if you do not use _rename_, it always makes it non-hygine.



### [§3] Arithmetic operations

###### [!Function] `+.`  _z_ _..._
###### [!Function] `*.`  _z_ _..._
###### [!Function] `-.`  _z_ _..._
###### [!Function] `-.`  _z1_ _z2_ _..._
###### [!Function] `/.`  _z_ _..._
###### [!Function] `/.`  _z1_ _z2_ _..._

The same as `+`, `*`, `-` and `/`. The difference is
these procedures converts given arguments inexact number.

###### [!Function] `mod-inverse`  _x_ _m_

_x_ and _m_ must be exact integer.

Returns `_x_ ^ -1 mod _m_`

###### [!Function] `mod-expt`  _x_ _e_ _m_

_x_, _e_ and _m_ must be exact integer.

Returns `_x_ ^ _e_ mod _m_`

### [§3] File system operations

###### [!Function] `file-size-in-bytes`  _filename_

Returns file size of filename in bytes. If filename does not exist, it
raises `&i/o` condition.


###### [!Function] `file-regular?`  _filename_
###### [!Function] `file-directory?`  _filename_
###### [!Function] `file-symbolic-link?`  _filename_
###### [!Function] `file-readable?`  _filename_
###### [!Function] `file-writable?`  _filename_
###### [!Function] `file-executable?`  _filename_

Returns file type or permission of given _filename_.

###### [!Function] `file-stat-ctime`  _filename_
###### [!Function] `file-stat-mtime`  _filename_
###### [!Function] `file-stat-atime`  _filename_

Returns file statistics time in nano sec.

The `file-stat-ctime` procedure returns last change time of _filename_.

The `file-stat-mtime` returns last modified time of _filename_.

The `file-stat-atime` returns last accesse time of _filename_.


###### [!Function] `change-file-timestamp!`  _filename_ _atime_ _mtime_

Changes given _filename_'s access time and modify time.

_atime_ specifies the access time of the _filename_.

_mtime_ specifies the modify time of the _filename_.

If _atime_ and/or _mtime_ are time object, then the
setting timestamp is the absolute time represented by the arguments.

If _atime_ and/or _mtime_ are real number, then the
setting timestamp is the difference of the arguments from current time.

If _atime_ and/or _mtime_ are #f, then the timestamp is intact.

If _atime_ and/or _mtime_ are true value other then time or real
number, then the timestamp is set to current time.


###### [!Function] `change-file-mode`  _filename_ _mode_

Changes given _filename_'s file mode.

_mode_ must be a fixnum encoded with POSIX file permission.

NOTE: On Windows, only the owner permission is considered.


###### [!Function] `create-symbolic-link`  _old-filename_ _new-filename_

Creates symbolic link of _old-filename_ as _new-filename_.

###### [!Function] `rename-file`  _old-filename_ _new-filename_

Renames given _old-filename_ to _new-filename_.

If _old-filename_ does not exist, it raises `&i/o`.

If _new-filename_ exists, it overwrite the existing file.


###### [!Function] `create-directory`  _path_
###### [!Function] `delete-directory`  _path_

Creates/deletes given directory. If it fails, it raises condition
`&i/o`.


###### [!Function] `read-directory`  _path_

Reads directory and returns contents as a string list. If _path_ does
not exist, it returns #f.


###### [!Function] `copy-file`  _src_ _dst_ _:optional_ _overwrite_

_src_ and _dst_ must be string and _src_ must indicate
existing file path.

Copies given _src_ file to _dst_ and returns #t if it's copied otherwise
#f.

If optional argument _overwrite_ is #t then it will over write the file even
if it exists.


###### [!Function] `current-directory`  _:optional_ _path_

Returns current working directory.

If optional argument _path_ is given, the `current-directory` sets
current working directory to _path_ and returns unspecified value.


###### [!Function] `set-current-directory`  _path_

Sets current working directory to _path_.

###### [!Function] `build-path`  _path1_ _path2_

_path1_ and _path2_ must be string.

Concatenate given parameter with platform dependent path separator.


### [§3] Hashtables

###### [!Function] `make-equal-hashtable`  _:optional_ _k_ _weakness_
###### [!Function] `make-string-hashtable`  _:optional_ _k_ _weakness_

Creates a hashtable.

The same as `make-eq-hashtable` and
`make-eqv-hashtable`. It uses `equal?` or `string=?` as
comparing procedure, respectively.

###### [!Function] `hashtable-values`  _hashtable_

Returns all _hashtable_'s values. This procedure is for consistancy of
`hashtable-keys`.

###### [!Function] `hashtable-keys-list`  _hashtable_
###### [!Function] `hashtable-values-list`  _hashtable_

Returns given _hashtable_'s _keys_ or _values_, respectively.

The R6RS required procedure `hashtable-keys` and `hashtable-values`are implemented with these procedures.


###### [!Function] `hashtable-type`  _hashtable_

Returns _hashtable_'s hash type as a symbol. The possible return
values are `eq`, `eqv`, `equal`, `string` and
`general`.


###### [!Function] `hashtable-weakness`  _hashtable_
 

Returns _hashtable_'s hash weakness as a symbol if the given
_hashtable_ is weak hashtable, otherwise #f. The possible return values are
`key`, `value` and `both`.  

### [§3] I/O

###### [!Function] `port-closed?`  _port_

Returns #t if given _port_ is closed, otherwise #f.

###### [!Function] `put-u16`  _out_ _v_ _endian_
###### [!Function] `put-s16`  _out_ _v_ _endian_
###### [!Function] `put-u32`  _out_ _v_ _endian_
###### [!Function] `put-s32`  _out_ _v_ _endian_

_out_ must be binary output port. _endian_ must be a value
returned from `endianness` macro.

Write _v_ to _out_ as unsigned/signed 16/32 bit integer.

###### [!Function] `get-u16`  _in_ _endian_
###### [!Function] `get-s16`  _in_ _endian_
###### [!Function] `get-u32`  _in_ _endian_
###### [!Function] `get-s32`  _in_ _endian_

_in_ must be binary input port. _endian_ must be a value
returned from `endianness` macro.

Read a number from _in_ as unsigned/signed 16/32.

NOTE: above `put-*` and `get-*` have only 16 and 32 bit integers.
This is because benchmark told us there's not much difference between C and
Scheme implementation. We may add 64 bit integers and floting number 
versions in future if there's enough demand.

###### [!Function] `read/ss`  _:optional_ _(port_ `(current-input-port)` _)_
###### [!Function] `write/ss`  _obj_ _:optional_ _(port_ `(current-output-port)` _)_

[SRFI-38] The `read/ss` reads a datum from given _port_.

The `write/ss` writes _obj_ to given _port_.

These are the same as `read` and `write` procedure, but it can handle
circular list.


###### [!Function] `format`  _port_ _string_ _arg_ _..._
###### [!Function] `format`  _string_ _arg_ _..._

[SRFI-28+] Formats _arg_ accoding to _string_. _Port_specifies the destination; if it is an output port, the formatted result is
written to it; if it is #t, the result is written to current output port; if it
is #f, the formatted result is returned as a string. _Port_ can be omitted
and the result is the same as when #f is given.

_String_ is a string that contains format directives. A format directive is
a character sequence begins with tilda `'~'`, and ends with some specific
characters. A format directive takes the corresponding arg and formats it. The
rest of string is copied to the output as is.

``(format #f "the answer is ~a" 48)`` => ``the anser is 48``

The format directive can take one or more parameters, separated by comma
characters. A parameter may be an integer or a character; if it is a character, 
it should be preceded by a quote character. Parameter can be omitted, in such
case the system default value is used. The interpretation of the parameters
depends on the format directive.

Furthermore, a format directive can take two additional flags: atmark
`'@'` and colon `':'`. One or both of them may modify the
behaviour of the format directive. Those flags must be placed immediately
before the directive character.

The following complete list of the supported directives. Either upper case or
lower case character can be used for the format directive; usually they have no
distinction, except noted.

~_mincol_,_colinc_,_minpad_,_padchar_,_maxcol_**A**
: Ascii output. The corresponding argument is printed by `display`. If an
  integer _mincol_ is given, it specifies the minimum number of characters to
  be output; if the formatted result is shorter than _mincol_, a whitespace is
  padded to the right (i.e. the result is left justified).
  The _colinc_, _minpad_ and _padchar_ parameters control, if given,
  further padding. A character padchar replaces the padding character for the
  whitespace. If an integer _minpad_ is given and greater than 0, at least
  _minpad_ padding character is used, regardless of the resulting width. If an
  integer _colinc_ is given, the padding character is added (after minpad) in
  chunk of _colinc_ characters, until the entire width exceeds _mincol_.
  If atmark-flag `'@'` is given, the format result is right
  justified, i.e. padding is added to the left.
  The _maxcol_ parameter, if given, limits the maximum number of characters to
  be written. If the length of formatted string exceeds _maxcol_, only
  _maxcol_ characters are written. If colon-flag is given as well and the
  length of formatted string exceeds _maxcol_, _maxcol_ - 4 characters
  are written and a string `" ..."` is attached after it.
  ``(format #f "|~a|" "oops")`` => ``|oops|``
  ``(format #f "|~10a|" "oops")`` => ``|oops      |``
  ``(format #f "|~10@a|" "oops")`` => ``|      oops|``
  ``(format #f "|~10,,,'*@a|" "oops")`` => ``|\*\*\*\*\*\*oops|``
  ``(format #f "|~,,,,10a|" '(abc def ghi jkl))`` => ``|abc def gh|``
  ``(format #f "|~,,,,10:a|" '(abc def ghi jkl))`` => ``|abc de ...|``

~_mincol_,_colinc_,_minpad_,_padchar_,_maxcol_**S**
: S-expression output. The corresponding argument is printed by `write`. The
  semantics of parameters and flags are the same as **~A** directive.
  ``(format #f "|~s|" "oops")`` => ``|"oops"|``
  ``(format #f "|~10s|" "oops")`` => ``|"oops"    |``
  ``(format #f "|~10@s|" "oops")`` => ``|    "oops"|``
  ``(format #f "|~10,,,'*@s|" "oops")`` => ``|\*\*\*\*"oops"|``

~_mincol_,_padchar_,_commachar_,_interval_**D**
: Decimal output. The argument is formatted as an decimal integer. If the argument
  is not an integer, all parameters are ignored and it is formatted by **~A**directive.
  If an integer parameter _mincol_ is given, it specifies minimum width of the
  formatted result; if the result is shorter than it, _padchar_ is padded on
  the left (i.e. the result is right justified). The default of padchar is a
  whitespace.
  ``(format #f "|~d|" 12345)`` => ``|12345|``
  ``(format #f "|~10d|" 12345)`` => ``|     12345|``
  ``(format #f "|~10,'0d|" 12345)`` => ``|0000012345|``
  If atmark-flag is given, the sign `'+'` is printed for the positive
  argument.
  If colon-flag is given, every _interval_th digit of the result is grouped
  and commachar is inserted between them. The default of _commachar_ is
  `','`, and the default of _interval_ is 3.
  ``(format #f "|~:d|" 12345)`` => ``|12,345|``
  ``(format #f "|~,,'_,4:d|" -12345678)`` => ``|-1234_5678|``

~_mincol_,_padchar_,_commachar_,_interval_**B**
: Binary output. The argument is formatted as a binary integer. The semantics of
  parameters and flags are the same as the **~D** directive.

~_mincol_,_padchar_,_commachar_,_interval_**O**
: Octet output. The argument is formatted as a octal integer. The semantics of
  parameters and flags are the same as the **~D** directive.

~_mincol_,_padchar_,_commachar_,_interval_**X**
~_mincol_,_padchar_,_commachar_,_interval_**x**
: Hexadecimal output. The argument is formatted as a hexadecimal integer. If
  `'X'` is used, upper case alphabets are used for the digits larger than 10.
  If `'x'` is used, lower case alphabets are used. The semantics of
  parameters and flags are the same as the **~D** directive.
  ``(format #f "~8,'0x" 259847592)`` => ``0f7cf5a8``
  ``(format #f "~8,'0X" 259847592)`` => ``0F7CF5A8``

Note: The format procedure's implementation and most of documentation is quoted
from Gauche.


###### [!Function] `port-ready?`  _:optional_ _(port_ _(current-input-port))_

Returns #t when port data are ready, otherwise #f.

If the given _port_ implementation does not support this functionality,
the return value will be always #t. Following example describes when this
always returns #t;

``````````scheme
;; Assume read! is provided.
(define user-port (make-custom-binary-input-port "my-port" read! #f #f))
(port-ready user-port)
``````````
=> ``#t``



###### [!Function] `buffered-port`  _port_ _buffer-mode_ _:key_ _buffer_

_port_ must be binary port.

Converts given _port_ to buffered port if the _port_ is not
buffered port.

Buffered port is a type of port which does not read or write immediately
but uses internal buffer. For example, file port with 
`(buffer-mode block)` or `(buffer-mode line)` uses the buffered
port. This is useful when actual I/O is more expensive than memory access.

The _buffer-mode_ must be a symbol the macro `buffer-mode` can
return. If the `buffer-mode` is `none`, then the procedure does not
convert the given _port_.

If the keyword argument _buffer_ is specified, it must be
a bytevector, then the converted buffered port uses specified bytevector
as its internal buffer. If the bytevector size is zero or literal bytevector
then `&assertion` is raised.


###### [!Function] `make-codec`  _symbol_ _getc_ _putc_ _data_

Creates a custom codec. _Symbol_ is the name of the codec. _Getc_and _putc_ must be procedures. _Data_ is an user data used in _getc_and _putc_.

_Getc_ must take 4 arguments, _input-port_, _error-handling-mode_,
_check-bom?_ and _userdata_. _Input-port_ is binary input port.
_Error-handling-mode_ is symbol can be `ignore`, `raise` or
`replace` depending on a transcoder which uses this custom codec.
_Check-bom?_ is boolean, if _getc_ is being called first time, it is #t,
otherwise #f. _Userdata_ is user defined data which is given when the codec
is created.

The basic process of _getc_ is reading binary data from _input-port_ and
converting the data to UCS4. Returning UCS4 must be integer and does not have to
be 4 byte.

_Putc_ must take 4 arguments, _output-port_, _char_, 
_error-handling-mode_ and _userdata_. _Output-port_ is binary output
port. _Char_ is character object which needs to be converted from UCS4.
_Error-handling-mode_ is symbol can be `ignore`, `raise` or
`replace` depending on a transcoder which uses this custom codec.
_Userdata_ is user defined data which is given when the codec is created.

The basic process of _putc_ is converting given UCS4 charactner to target
encoding data and putting it to _output-port_ as binary data.

For sample implementation, see sitelib/encoding directory. You can find some
custom codecs.


### [§3] Symbols

###### [!Function] `symbol<?`  _symbol1_ _symbol2_ _symbols_ _..._
###### [!Function] `symbol<=?`  _symbol1_ _symbol2_ _symbols_ _..._
###### [!Function] `symbol>?`  _symbol1_ _symbol2_ _symbols_ _..._
###### [!Function] `symbol>=?`  _symbol1_ _symbol2_ _symbols_ _..._

Return #t if given _symbol1_ is less than, less than or equal to,
greater than or greater than or equal to _symbol2_. If _symbols_ are
not null, then the procedures continue to compare until the given arguments
are exhausted.

These procedures are analogy of `string<?`, `string<=?`,
`string>?` and `string>=?`.

These can also be implemented as follows:
``(define (symbol<? . syms) (apply string<? (map symbol->string syms)))``

However, this build-in version won't converts given symbols to strings.


### [§3] Keywords

Sagittarius has keyword objects which starts with `':'`. It has almost the
same feature as symbol, however it can not be bounded with any values. The 
keyword objects are self quoting so users don't have to put `'` explicitly.

The keyword notation is \*NOT\* available on R6RS or R7RS reader mode. Thus
`#!r6rs` or `#!r7rs` directive and `-r6` or `-r7`command line option disable it.

###### [!Function] `make-keyword`  _symbol_
###### [!Function] `symbol->keyword`  _symbol_

Creates a new keyword from _symbol_.

###### [!Function] `string->keyword`  _string_

Creates a new keyword from _string_.

###### [!Function] `keyword?`  _obj_

Returns #t if _obj_ is keyword, otherwise #f.


###### [!Function] `keyword->symbol`  _keyword_

Returns a symbol representation of given keyword _keyword_.

###### [!Function] `keyword->string`  _keyword_

Returns a string representation of given keyword _keyword_.

###### [!Function] `get-keyword`  _keyword_ _list_ _:optional_ _fallback_

Returns the element after given _keyword_ from given _list_.

The elements count of the _list_ should be even number, otherwise the
procedure might raise `&error` when _keyword_ is not found in
_list_.

If _fallback_ is given and the procedure could not find the _keyword_from the _list_, the _fallback_ will be return. Otherwise it raises
`&error`.

``(get-keyword :key '(a b c :key d))`` => ``d``

``(get-keyword :key '(a b c d e))`` => ``&error``

``(get-keyword :key '(a b c d e) 'fallback)`` => ``&error``

``(get-keyword :key '(a b c d e f) 'fallback)`` => ``fallback``



### [§3] Weak box

A weak  box is a reference  to an object  that doesn’t prevent the  object from
being garbage-collected.

###### [!Function] `weak-box?`  _obj_

Returns #t if _obj_ is weak box otherwise #f.

###### [!Function] `make-weak-box`  _obj_

Returns a weak box whose value is _obj_.

###### [!Function] `weak-box-empty?`  _wb_

Returns #t if the value of given weak box _wb_ is garbage collocted.
Otherwise #f.

###### [!Function] `weak-box-ref`  _wb_ _:optional_ _(fallback_ _#f)_

Returns the value of given weak box _wb_ if it's not garbage 
collocted. Otherwise returns _fallback_.

###### [!Function] `weak-box-set!`  _wb_ _obj_

Replace the value of given _wb_ with _obj_.

### [§3] Weak vector

A weak vector is like a vector of objects, except each object can be garbage
collected if it is not referenced from objects other than weak vectors.  If the
object is collected, the entry of the weak vector is replaced to #f.

###### [!Function] `weak-vector?`  _obj_

Returns #t if _obj_ is weak vector otherwise #f.

###### [!Function] `make-weak-vector`  _size_

Creates and returns a weak vector of size _size_.

###### [!Function] `weak-vector-length`  _wvec_

Returns the length of given weak vector _wvec_

###### [!Function] `weak-vector-ref`  _wvec_ _k_ _:optional_ _fallback_

Returns _k_-th element of a weak vector _wvec_.

By default, `weak-vector-ref` raise an `&assertion` if _k_ is
negative, or greater than or equal to the size of _wvec_. However, if an
optional argument _fallback_ is given, it is returned for such case.

If the element has been garbage collected, this procedure returns _fallback_if it is given, #f otherwise. 


###### [!Function] `weak-vector-set!`  _wvec_ _k_ _value_

Sets _k_-th element of the weak vector _wvec_ to _value_. It
raises an `&assertion` if _k_ is negative or greater than or equal to
the size of _wvec_. 


### [§3] Weak hashtable

A weak hashtable contains references to an object that doesn't prevent the
object from being garbage-collected the same as weak vector. A weak hashtable
is like a hashtable, except each entry can be garbage collected if it is not
referenced from objects other than weak hashtable according to the constructed
condition.

###### [!Function] `weak-hashtable?`  _obj_

Returns #t if _obj_ is weak hashtable otherwise #f.

###### [!Function] `make-weak-eq-hashtable`  _:key_ _(init-size_ _200)_ _(weakness_ _'both)_ _default_
###### [!Function] `make-weak-eqv-hashtable`  _:key_ _(init-size_ _200)_ _(weakness_ _'both)_ _default_

Make a weak hashtable with the hash procedure `eq?` and `eqv?`,
respectively.

The keyword argument _init-size_ specifies the initial size of the
weak hashtable.

The kwyword argument _weakness_ specifies the place where the weak pointer
is. This must be one of the `key`, `value` and `both`. If the
`key` is specified, then weak hashtable's weak pointer will be the key
so is `value`. If the `both` is specified, then it will be both.

The keyword argument _default_ specifies the default value when the entry
is garbage collected. If this is not spceified, then undefined value will be
used.


###### [!Function] `weak-hashtable-ref`  _weak-hashtable_ _key_ _:optional_ _(default_ _#f)_

Returns the value in _weak-hashtable_ associated with _key_. If
_weak-hashtable_ does not contain an association for _key_, 
_default_ is returned.


###### [!Function] `weak-hashtable-set!`  _weak-hashtable_ _key_ _obj_

Changes _weak-hashtable_ to associate _key_ with _obj_,
adding a new association or replacing any existing association for _key_,
and returns unspecified values.


###### [!Function] `weak-hashtable-delete!`  _weak-hashtable_ _key_

Removes any association for _key_ within _weak-hashtable_ and
returns unspecified values.


###### [!Function] `weak-hashtable-keys-list`  _weak-hashtable_
###### [!Function] `weak-hashtable-values-list`  _weak-hashtable_

Returns a list of keys and values in the given _weak-hashtable_,
respectively.


###### [!Function] `weak-hashtable-copy`  _weak-hashtable_

Returns a copy of _weak-hashtable_.

###### [!Function] `weak-hashtable-shrink`  _weak-hashtable_

Shrink the given _weak-hashtable_ and returns the number of
removed entry. This is only for GC friendliness.

### [§3] Bytevector operations

###### [!Function] `bytevector->sinteger`  _bytevector_ _:optional_ _start_ _end_
###### [!Function] `bytevector->uinteger`  _bytevector_ _:optional_ _start_ _end_
###### [!Function] `bytevector->integer`  _bytevector_ _:optional_ _start_ _end_

Converts given bytevector _bytevector_ to exact integer. If optional
argument _start_ is given, conversion starts with index _start_. If
optional argument _end_ is given, convertion ends by index _end_. The
conversion only happens in big endian. 

The first form converts to signed integer and the rest convert to unsigned
integer.


###### [!Function] `sinteger->bytevector`  _ei_ _:optional_ _size_
###### [!Function] `uinteger->bytevector`  _ei_ _:optional_ _size_
###### [!Function] `integer->bytevector`  _ei_ _:optional_ _size_

_Ei_ must be exact integer. Converts exact integer _ei_ to a
bytevector.

The first form can accept signed integer and converts with two's complement
form. The rest forms only accept unsigned integer and simply convert to
bytes.

If optional argument _size_ is given, then the procedure returns _size_size bytevector.

NOTE: The conversion is processed from right most byte so if the _size_ is
smaller than given _ei_ bytes, then the rest of left bytes will be dropped.

NOTE: the endianness is always big endianness.

``(integer->bytevector #x12345678 5)`` => ``#vu8(#x00 #x12 #x34 #x56 #x78)``

``(integer->bytevector #x12345678 3)`` => ``#vu8(#x34 #x56 #x78)``



###### [!Function] `bytevector-append`  _bvs_ _..._

Returns a newly allocated bytevector that contains all elements in
order from the subsequent locations in _bvs ..._.


###### [!Function] `bytevector-concatenate`  _list-of-bytevectors_

Appends each bytevectors in _list-of-bytevectors_. This is
equivalent to:

``(apply bytevector-append _list-of-bytevectors_)``



### [§3] List operations

###### [!Function] `circular-list?`  _list_
###### [!Function] `dotted-list?`  _list_

[SRFI-1] Returns #t if _list_ is circular or dotted list,
respectively. Otherwise #f.

###### [!Function] `acons`  _obj1_ _obj2_ _obj3_

Returns `(cons (cons _obj1_ _obj2_) _obj3_)`. Useful to
put an entry at the head of an associative list.


###### [!Function] `append!`  _list_ _..._

[SRFI-1] Returns a list consisting of the elements of the first _list_followed by the elements of the other _lists_. The cells in the lists except
the last one may be reused to construct the result. The last argument may be any
object.


###### [!Function] `reverse!`  _list_ _..._

[SRFI-1] Returns a list consisting of the elements of _list_ in
reverse order. The cells of list may be reused to construct the returned list.


### [§3] Vector operations

###### [!Function] `vector-copy`  _vector_ _:optional_ _start_ _end_ _fill_

[SRFI-43] Copies a vector _vector_. Optional _start_ and _end_arguments can be used to limit the range of _vector_ to be copied. If the
range specified by _start_ and _end_ falls outside of the original
vector, the _fill_ value is used to fill the result vector.

``(vector-copy '#(0 1 2 3 4) 1 4)`` => ``#(1 2 3)``

``(vector-copy '#(0 1 2 3 4) 4 6 #f)`` => ``#(4 #f)``

``(vector-copy '#() 0 2 #t)`` => ``#(#t #t)``



###### [!Function] `vector-append`  _vector_ _..._

[SRFI-43] Returns a newly allocated vector that contains all elements in
order from the subsequent locations in _vector ..._.

``(vector-append '#(1 2) '#(a b) '#(5))`` => ``#(1 2 a b 5)``



###### [!Function] `vector-concatenate`  _list-of-vectors_

[SRFI-43] Appends each vectors in _list-of-vectors_. This is
equivalent to:

``(apply vector-append _list-of-vectors_)``



###### [!Function] `vector-reverse`  _vector_ _:optional_ _start_ _end_
###### [!Function] `vector-reverse!`  _vector_ _:optional_ _start_ _end_

[SRFI-43] Reverse the given _vector_'s elements.

The second form reverses the given _vector_ destructively.

Optional arguments _start_ and _end_ must be non negative integer
if it's given. And it restricts the range of the target elements.


### [§3] String operations

###### [!Function] `string-scan`  _string_ _item_ _:optional_ _return_

Scan _item_ (either a string or a character) in _string_.

The _return_ argument specified what value should be returned when
_item_ is found in _string_. It must be one of the following symbols;

`index`
: Returns the index in _string_ if _item_ is found, or `#f`.
  This is the default behaviour.

`before`
: Returns a substring of _string_ before _item_, or `#f` if
  _item_ is not found.

`after`
: Returns a substring of _string_ after _item_, or `#f` if
  _item_ is not found.

`before*`
: Returns a substring of _string_ before _item_, and the substring
  after it. If _item_ is not found then return `(values #f #f)`.

`after*`
: Returns a substring of _string_ up to the end of _item_, and the
  rest. after it. If _item_ is not found then return 
  `(values #f #f)`.

`both`
: Returns a substring of _string_ before _item_ and after
   _item_. If _item_ is not found, return `(values #f #f)`.



###### [!Function] `string-concatenate`  _list-of-strings_

[SRFI-13] Appends each strings in _list-of-strings_. This is
equivalent to:

``(apply string-append _list-of-strings_)``



###### [!Function] `istring?`  _obj_

Returns #t if the given _obj_ is an immutable string, otherwise #f.

Immutable strings are one of the followings:

- String literals. (i.e. `"abc"`)
- Strings converted to immutable string by `string->istring`



###### [!Function] `string->istring`  _string_ _:optional_ _start_ _end_

Returns copy of the given _string_. The returning string is
converted to immutable string.

The optional argument _start_ and _end_ are given, then the
returing string is contains only the range of given _string_.


### [§3] Load path

###### [!Function] `add-load-path`  _path_ _:key_ _(append_ _#f)_

Add given _path_ to current loading path list.

If keyword argument _append_ is `#t`, then given _path_ is
appended to the current loading path list, otherwise prepend to the list.


###### [!Function] `add-load-suffix`  _suffix_ _:key_ _(append_ _#f)_

Add given _suffix_ to current suffix list.

If keyword argument _append_ is `#t`, then given _suffix_ is
appended to the current suffix list, otherwise prepend to the list.

The _suffix_ should contain `.` as well.


###### [!Function] `load-path` 
###### [!Function] `load-path`  _paths_

Retrives/sets current loading path.

If the latter form is used, then the given path list _paths_ is set
to current loading path list. Otherwise retrieves the list.

The returning value is not copied, thus modifying the list may cause
unexpected results.


### [§3] System and platform

###### [!Function] `uname` 

Return a vector which contains the following information.

System name
: Operating system name. E.g. "Linux"

Node name
: The name of the computer.

Release
: Release number of the system if availabel.

Version
: Version number of the system if availabel.

Machine
: Machine architecture of the system.

On POSIX environment, this is the result of `uname(2)`.

On Windows environment, this is the result of couple of Win32 API calls
and formatted to look like the result of `uname(2)`.


###### [!Function] `get-mac-address`  _:optional_ _(position_ _0)_

Retrieves MAC address. The returning value is a bytevector which
contains 6 elements.

If optional argument specifies which NIC's MAC address should be returned.


###### [!Function] `get-process-times` 
###### [!Function] `get-thread-times`  _:optional_ _thread_

Returns a vector having 3 elements; _user time_, _system time_and _tick_.

The first procedure returns process CPU time, the second procedure returns
thread CPU time. If the optional argument _thread_ is given, then the
procedure retrieves CPU time of given thread. Otherwise current thread.

_tick_ is a clock tick. Users can compute CPU second Dividing
_user time_ or _system time_ by this value.

NOTE: _tick_ may not be the same even on the same platform. For example,
`get-process-times` returns value of `_SC_CLK_TCK`, and
`get-thread-times` returns 1000000 (on OSX), 1000000000 (on other POSIX
environments which have `pthread_getcpuclockid`), or `_SC_CLK_TCK`(on other POSIX environments which don't have `pthread_getcpuclockid`). So
users must not assume the value is one of them and calculate CPU second with
assumed value.


###### [!Function] `gc` 

Invokes garbage collection manually.

###### [!Function] `cpu-count` 

Returns number of CPUs. The result contains hyperthreading, for example,
if the CPU is Core i5 which have 2 cores and hyperthreading is enabled, then
the returning value is 4.

On Linux environment, the procedure also considers `taskset(1)`.

This procedure returns static value initialised during initialisation of
Sagittarius process. Thus, if the process is restricted after its 
initialisation by `taskset(1)`, for example the environment has 4 CPUs
but restricted to 1, then this process, however, returns 4.


### [§3] Debugging aid

###### [!Function] `disasm`  _closure_

Disassembles the compiled body of _closure_ and print it.


###### [!Function] `arity`  _procedure_

Returns arity of given _procedure_.

It returns a pair whose car part is required arguments count and cdr part
is boolean which indicates if the procedure accepts optional arguments or
not.


###### [!Library] `(time)` 

Exports `time` macro

###### [!Macro] `time`  _expr_

Evaluate _expr_ and shows time usage.

The macro return the result of _expr_.

* @[[sagittarius/control.md](sagittarius/control.md)]
* @[[sagittarius/ffi.md](sagittarius/ffi.md)]
* @[[sagittarius/filewatch.md](sagittarius/filewatch.md)]
* @[[sagittarius/io.md](sagittarius/io.md)]
* @[[sagittarius/mop.md](sagittarius/mop.md)]
* @[[sagittarius/object.md](sagittarius/object.md)]
* @[[sagittarius/process.md](sagittarius/process.md)]
* @[[sagittarius/reader.md](sagittarius/reader.md)]
* @[[sagittarius/record.md](sagittarius/record.md)]
* @[[sagittarius/regex.md](sagittarius/regex.md)]
* @[[sagittarius/socket.md](sagittarius/socket.md)]
* @[[sagittarius/threads.md](sagittarius/threads.md)]
* @[[sagittarius/timezone.md](sagittarius/timezone.md)]
* @[[sagittarius/debug.md](sagittarius/debug.md)]
* @[[sagittarius/generators.md](sagittarius/generators.md)]
