[§2] (rsa pkcs :12) - PKCS#12 {#pkcs.12}
-------------

This section describes the implementation of PKCS#12 specification library.
However we do not describe PKCS#12 itself and I don't think it is necessary to
know it if you just want to use it.

###### [!Library] `rsa pkcs :12` 

This library provides procedures for PKCS#12 operations.

### [§3] Keystore APIs

The example shows how to use keystore APIs:

``````````scheme
(import (rnrs) (rsa pkcs :12))

(define keystore (load-pkcs12-keystore-file "keystore.p12" "pass"))

(pkcs12-keystore-get-key keystore "key-name" "key-pass")
;; -> <private-key>

(pkcs12-keystore-get-certificate keystore "cert-name")
;; -> <x509-certificate>

;; certs are list of X509 certificate associated with given private-key
(pkcs12-keystore-set-key! keystore "key-name2" private-key "key-pass2" certs)

;; cert must be an X509 certificate
(pkcs12-keystore-set-certificate! keystore "cert-name2" cert)

(store-pkcs12-keystore-to-file keystore "keystore2.p12" "pass2")
``````````

###### [!Class] `<pkcs12-keystore>` 

PKCS#12 keystore class.

###### [!Function] `pkcs12-keystore?`  _obj_

Returns #t if given _obj_ is PKCS#12 keystore object, otherwise #f.

###### [!Function] `make-pkcs12-keystore` 

Returns newly created PKCS#12 keystore object.

###### [!Function] `load-pkcs12-keystore`  _input-port_ _storepass_
###### [!Function] `load-pkcs12-keystore-file`  _file_ _storepass_

Loads PKCS#12 keystore from given _input-port_ or _file_ and
returns newly created keystore object.

The `load-pkcs12-keystore` loads from given binary input port.

The `load-pkcs12-keystore-file` loads from given file.

_storepass_ must be a string and a password for given keystore.


###### [!Function] `pkcs12-keystore-get-key`  _keystore_ _alias_ _keypass_

Retrives a private key associated with _alias_ from _keystore_.
If there is no key entry associated with _alias_ then #f is returned.

_alias_ must be a string.

_keypass_ must be a string. It is used to decrypt the private key and
it is **not** allowed to pass empty password.

###### [!Function] `pkcs12-keystore-get-certificate`  _keystore_ _alias_

Retrives a certificate associated with _alias_ from _keystore_.
If there is no certificate entry associated with _alias_ then #f is 
returned.

_alias_ must be a string.


###### [!Function] `pkcs12-keystore-get-certificate-chain`  _keystore_ _alias_

Retrives certificate chain associated with given key alias _alias_from _keystore_. If there is no certificate chain then `'()` is
returned.

_alias_ must be a string.


###### [!Function] `pkcs12-keystore-contains-alias?`  _keystore_ _alias_

Returns #t if _keystore_ contains _alias_. Otherwise #f.

###### [!Function] `store-pkcs12-keystore`  _keystore_ _output-port_ _storepass_
###### [!Function] `store-pkcs12-keystore-to-file`  _keystore_ _file_ _storepass_

Writes given _keystore_ to _output-port_ or _file_.

The `store-pkcs12-keystore` writes to given binary output port
_output-port_.

The `store-pkcs12-keystore-to-file` writes to given file _file_.

_storepass_ must be a string and is used to encrypt whole contents.


###### [!Function] `pkcs12-keystore-set-key!`  _keystore_ _alias_ _private-key_ _key-pass_ _certs_

_alias_ must be a string represents the name of _private-key_ in
the _keystore_.

_private-key_ must be an RSA private key.

_key-pass_ must be a string and is used to encrypt given _private-key_.

_certs_ must be a list of X509 certificates which associated with
_private-key_.

Stores given _private-key_ to _keystore_.

The implementation allows users to set separate password from storepass.
Be aware that current implementation of Bouncy Castle JCE uses the same
password as storepass to encrypt a private key. Thus if you use different
password, then it is not compatible with Bouncy Castle.


###### [!Function] `pkcs12-keystore-set-certificate!`  _keystore_ _alias_ _cert_

_alias_ must be a string represents the name of _cert_ in
_keystore_.

_cert_ must be an X509 certificate.

Stores given _cert_ to _keystore_.


###### [!Function] `pkcs12-keystore-delete-entry!`  _keystore_ _alias_

_alias_ must be a string.

Removes the entry associated with _alias_ in _keystore_.


###### [!Function] `pkcs12-keystore-aliases`  _keystore_

Returns all defined names in _keystore_.

