[§2] (rfc :5322) - Internet message format library {#rfc.5322}
-------------

###### [!Library] `(rfc :5322)` 

This library provides the procedures for internet message format defined
in RFC5322([RFC 5322](http://tools.ietf.org/html/rfc5322)).


### [§3] Parsing message headers

###### [!Function] `rfc5322-read-headers`  _input-port_ _:key_ _(strict?_ _#f)_ _reader_

_input-port_ must be input port and will be passed to _reader_.

Reads RFC5322 format message from the given port until it reaches the end of the
message header and returns a list of the following format;

``((name body) ...)``

_name ..._ are the field names and _body ..._ are the
correspoinding field body. Both are as string. Field names are converted to
lower-case characters.

The keyword argument _strict?_ switches the behaviour when the procedure
encounter EOF. If it's #f, then it simply ignore the field and return the
composed list. And if it's #t, it raises `&rfc5322-parse-error`.

The keyword argument _reader_ reads a line from given port. The default
is `rfc5322-line-reader` and it treats both LF and CRLF eof style. If you
want to read from binary port, you need to pass own reader.


###### [!Function] `rfc5322-line-reader`  _input-port_

_input-port_ must be textual input port.

Reads a line from given port. If the last character is CR chop it off.


###### [!Function] `rfc5322-header-ref`  _header-list_ _field-name_ _._ _maybe-default_

An utility procedure to get a specific field from the parsed header list,
which is returned by `rfc5322-read-headers`.

If the field with given _field-name_ is in _header-list_, the
procedure returns its value in a string.  Otherwise, if default is
given, it is returned, and if not, #f is returned.


###### [!Function] `rfc5322-header-ref*`  _header-list_ _field-name_

An utility procedure to get a specific field from the parsed header list,
which is returned by `rfc5322-read-headers`.

This procedure collects all the given _field-name_ from the
_header-list_. If there's no header named _field-name_, then the
procedure returns `()`.


### [§3] Basic field parsers

###### [!Function] `rfc5322-next-token`  _input-port_ _:optional_ _tokenizer-spec_

_input-port_ must be textual input port.

A basic tokenizer. First it skips whitespaces and/or comments (CFWS) from 
_input-port_, if any. Then reads one token according to 
var{tokenizer-specs}. If _input-port_ reaches EOF before any token is read,
EOF is returned.

_tokenizer-specs_ is a list of tokenizer spec. which is a cons of a char-set
and a procedure.

After skipping CFWS, the procedure peeks a character at the head of
_input-port_, and checks it against the char-sets in _tokenizer-specs_one by one. If a char-set that contains the character belongs to is found, then
a token is retrieved with calling the procedure with _input-port_ to read a
token.

If the head character doesn’t match any char-sets, the character is taken from
_input-port_ and returned.

The default _tokenizer-specs_ is as follows:
 	
``(list (cons (string->char-set ":") rfc5322-quoted-string)
      (cons *rfc5322-atext-chars* rfc5322-dot-atom))
``



###### [!Function] `rfc5322-field->tokens`  _field_ _:optional_ _tokenizer-spec_

A convenience procedure. Creates a string input port from given
_field_ and calls `rfc5322-next-token` repeatedly on it until it
consumes all input, and returns a list of tokens.


###### [!Function] `rfc5322-skip-cfws`  _input-port_

Consumes whitespace characters and/or any comments from _input-port_and returns a non comment and whitespace character. The returned character
remains in _input-port_.


###### [!Variable] `*rfc5322-atext-chars*` 

A character set which is defined RFC 5322 section 3.2.3 Atom.

###### [!Variable] `*rfc5322-standard-tokenizers*` 

Default tokenizer.

###### [!Function] `rfc5322-dot-atom`  _input-port_
###### [!Function] `rfc5322-quoted-string`  _input-port_

Tokenizers for `dot-atom` and `quoted-string` respectively.

### [§3] Specific field parsers

###### [!Function] `rfc5322-parse-date`  _string_

Takes RFC-5322 type date string and returns eight values:

`year, month, day-of-month, hour, minute, second, time-zone, day-of-week.`_time-zone_ is an offset from UT in minutes. _day-of-week_ is a day from
sunday, and may be #f if that information is not available. _month_ is an
integer between 1 and 12, inclusive. If the _string_ is not parsable, all
the elements are #f. 


### [§3] Message constructors

###### [!Function] `rfc5322-write-headers`  _headers_ _:key_ _(output_ _(current-output-port))_ _
_ _(check_ _:error)_ _(continue_ _#f)_

Writes the given _header_ to the port _output_.


###### [!Function] `date->rfc5322-date`  _date_

_date_ must be SRFI-19 date.

Returns RFC 5322 date formatted string.


